'use strict';

var vutils = require('@visactor/vutils');
var base = require('./base.js');
var enums = require('../../enums.js');
var bezierUtils = require('../../bezier-utils.js');

class CubicBezierCurve extends base.Curve {
  constructor(p0, p1, p2, p3) {
    super(), this.type = enums.CurveTypeEnum.CubicBezierCurve, this.p0 = p0, this.p1 = p1, this.p2 = p2, this.p3 = p3;
  }
  _validPoint() {
    return Number.isFinite(this.p0.x + this.p0.y + this.p1.x + this.p1.y + this.p2.x + this.p2.y + this.p3.x + this.p3.y);
  }
  getPointAt(t) {
    if (!1 !== this.defined) return bezierUtils.cubicPointAt(this.p0, this.p1, this.p2, this.p3, t);
    throw new Error("defined为false的点不能getPointAt");
  }
  calcLength() {
    return this._validPoint() ? bezierUtils.cubicLength(this.p0, this.p1, this.p2, this.p3) : 60;
  }
  calcProjLength(direction) {
    return direction === enums.Direction.ROW ? vutils.abs(this.p0.x - this.p3.x) : direction === enums.Direction.COLUMN ? vutils.abs(this.p0.y - this.p3.y) : 0;
  }
  getAngleAt(t) {
    const minT = vutils.max(t - .01, 0),
      maxT = vutils.min(t + .01, 1),
      minP = this.getPointAt(minT),
      maxP = this.getPointAt(maxT);
    return vutils.atan2(maxP.y - minP.y, maxP.x - minP.x);
  }
}

exports.CubicBezierCurve = CubicBezierCurve;
