'use strict';

var vutils = require('@visactor/vutils');
var base = require('./base.js');
var enums = require('../../enums.js');

class LineCurve extends base.Curve {
  constructor(p0, p1) {
    super(), this.type = enums.CurveTypeEnum.LineCurve, this.p0 = p0, this.p1 = p1;
  }
  getPointAt(t) {
    if (!1 !== this.defined) return vutils.PointService.pointAtPP(this.p0, this.p1, t);
    throw new Error("defined为false的点不能getPointAt");
  }
  getAngleAt(t) {
    return null == this.angle && (this.angle = vutils.atan2(this.p1.y - this.p0.y, this.p1.x - this.p0.x)), this.angle;
  }
  _validPoint() {
    return Number.isFinite(this.p0.x + this.p0.y + this.p1.x + this.p1.y);
  }
  calcLength() {
    return this._validPoint() ? vutils.PointService.distancePP(this.p0, this.p1) : 60;
  }
  calcProjLength(direction) {
    return direction === enums.Direction.ROW ? vutils.abs(this.p0.x - this.p1.x) : direction === enums.Direction.COLUMN ? vutils.abs(this.p0.y - this.p1.y) : 0;
  }
}

exports.LineCurve = LineCurve;
