'use strict';

var linear = require('./linear.js');
var basis = require('./basis.js');
var monotone = require('./monotone.js');
var step = require('./step.js');
var linearClosed = require('./linear-closed.js');
var catmullRom = require('./catmull-rom.js');
var catmullRomClose = require('./catmull-rom-close.js');

function calcLineCache(points, curveType, params) {
  var _a, _b;
  switch (curveType) {
    case "linear":
    default:
      return linear.genLinearSegments(points, params);
    case "basis":
      return basis.genBasisSegments(points, params);
    case "monotoneX":
      return monotone.genMonotoneXSegments(points, params);
    case "monotoneY":
      return monotone.genMonotoneYSegments(points, params);
    case "step":
      return step.genStepSegments(points, .5, params);
    case "stepBefore":
      return step.genStepSegments(points, 0, params);
    case "stepAfter":
      return step.genStepSegments(points, 1, params);
    case "catmullRom":
      return catmullRom.genCatmullRomSegments(points, null !== (_a = null == params ? void 0 : params.curveTension) && void 0 !== _a ? _a : .5, params);
    case "catmullRomClosed":
      return catmullRomClose.genCatmullRomClosedSegments(points, null !== (_b = null == params ? void 0 : params.curveTension) && void 0 !== _b ? _b : .5, params);
    case "linearClosed":
      return linearClosed.genLinearClosedSegments(points, params);
  }
}

exports.Linear = linear.Linear;
exports.genLinearSegments = linear.genLinearSegments;
exports.genLinearTypeSegments = linear.genLinearTypeSegments;
exports.Basis = basis.Basis;
exports.genBasisSegments = basis.genBasisSegments;
exports.genBasisTypeSegments = basis.genBasisTypeSegments;
exports.point = basis.point;
exports.MonotoneX = monotone.MonotoneX;
exports.MonotoneY = monotone.MonotoneY;
exports.genMonotoneXSegments = monotone.genMonotoneXSegments;
exports.genMonotoneXTypeSegments = monotone.genMonotoneXTypeSegments;
exports.genMonotoneYSegments = monotone.genMonotoneYSegments;
exports.genMonotoneYTypeSegments = monotone.genMonotoneYTypeSegments;
exports.Step = step.Step;
exports.genStepSegments = step.genStepSegments;
exports.genStepTypeSegments = step.genStepTypeSegments;
exports.LinearClosed = linearClosed.LinearClosed;
exports.genLinearClosedSegments = linearClosed.genLinearClosedSegments;
exports.genLinearClosedTypeSegments = linearClosed.genLinearClosedTypeSegments;
exports.calcLineCache = calcLineCache;
