import { genLinearSegments } from './linear.js';
export { Linear, genLinearTypeSegments } from './linear.js';
import { genBasisSegments } from './basis.js';
export { Basis, genBasisTypeSegments, point } from './basis.js';
import { genMonotoneYSegments, genMonotoneXSegments } from './monotone.js';
export { MonotoneX, MonotoneY, genMonotoneXTypeSegments, genMonotoneYTypeSegments } from './monotone.js';
import { genStepSegments } from './step.js';
export { Step, genStepTypeSegments } from './step.js';
import { genLinearClosedSegments } from './linear-closed.js';
export { LinearClosed, genLinearClosedTypeSegments } from './linear-closed.js';
import { genCatmullRomSegments } from './catmull-rom.js';
import { genCatmullRomClosedSegments } from './catmull-rom-close.js';

function calcLineCache(points, curveType, params) {
  var _a, _b;
  switch (curveType) {
    case "linear":
    default:
      return genLinearSegments(points, params);
    case "basis":
      return genBasisSegments(points, params);
    case "monotoneX":
      return genMonotoneXSegments(points, params);
    case "monotoneY":
      return genMonotoneYSegments(points, params);
    case "step":
      return genStepSegments(points, .5, params);
    case "stepBefore":
      return genStepSegments(points, 0, params);
    case "stepAfter":
      return genStepSegments(points, 1, params);
    case "catmullRom":
      return genCatmullRomSegments(points, null !== (_a = null == params ? void 0 : params.curveTension) && void 0 !== _a ? _a : .5, params);
    case "catmullRomClosed":
      return genCatmullRomClosedSegments(points, null !== (_b = null == params ? void 0 : params.curveTension) && void 0 !== _b ? _b : .5, params);
    case "linearClosed":
      return genLinearClosedSegments(points, params);
  }
}

export { calcLineCache, genBasisSegments, genLinearClosedSegments, genLinearSegments, genMonotoneXSegments, genMonotoneYSegments, genStepSegments };
